#!/usr/bin/perl
#
# http_errors.cgi - More intelligent error handling.
#
$VERSION = 0.7;
#
# COPYRIGHT 2001 - Rob <webmaster@rock13.com>
# You are free to distribute and deploy this script in its complete package
# so long as you do NOT profit from it in anyway other than intellectual
# gain. Configure the defaults and let the rest alone. If you don't like how
# it works--get a different script. If this script happens to break something
# the author assumes no responsibility. All comments must stay intact.
#
# NOTE
# This script does some things that Apache's mod_speling and mod_rewrite might
# do if you set them up for it. But they can add unnecessary overhead and your
# server guru may not want to set them up. Of course http_errors.cgi is no
# replacement for these either.
# - - - - - - - - - - - - - - - - - - -
# CONFIGURATION
# - - - - - - - - - - - - - - - - - - -
my $email  = 'you@example.com';		# Email displayed to users
my $css    = '';			# Path to an optional CSS file
my $title  = 'Error Processing';		# HTML Title for Error pages
# - - - - - - - - - - - - - - - - - - -
# END CONFIGURATION
# - - - - - - - - - - - - - - - - - - -
# Do not edit beyond this point unless you know what you are doing.
#
my $STATUS   = $ENV{REDIRECT_STATUS};
my $DOC_ROOT = $ENV{DOCUMENT_ROOT};
my $HOST     = $ENV{HTTP_HOST};
my $REFERER  = $ENV{HTTP_REFERER};
my $request  = lc($ENV{REQUEST_URI});
my $skip_mess = 0;
my $path = $request;

$path =~ s#[^/]+$##;

# Does the file, in lowercase, exist is readable and not a directory.
if (-e ($DOC_ROOT . $request) && -r _ && $STATUS ne 403) {
	my $url = 'http://' . $HOST . $request;
	print "Location: $url\n";
	print "Status: 302\n\n";
	exit;
}

# You can test for other conditions here and redirect accordingly.
# I used this one to rewrite a URL such as 'wwf_0199.html' to
# '9901.html'

# if ($request =~ m#wwf_\d+.html$#) {
#	my $url;
#	$request =~ s#wwf_(\d\d)(\d\d)\.html#$2$1\.html#;
#	$url = 'http://' . $HOST . $request;
#	print "Location: $url\n";
#	print "Status: 302\n\n";
#	exit;
# }


print <<BEGIN_HTML;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<title>$title</title>
BEGIN_HTML

print qq{<link rel="StyleSheet" href="$css" type="text/css">\n} if($css);

print <<HTML;
<!-- http_errors.cgi $VERSION http://rock13.com/webhelp/perl/http_errors/ -->
</head>
<body>
<h1>$title</h1>
HTML

if ($0 =~ m#$request$#) {
	# If the script itself is called display an information page
	about_me();
	$skip_mess = 1;

} elsif ($request =~ m#^/temp/.+$#) {

	temp_file($request);

} elsif ($request =~ m#(jpg|gif|png)$# && -e ($DOC_ROOT . $path)) {

	image_file($request);

} else {

	other_file($request);

}

my $message = "message.txt";
if(-e $message && ! $skip_mess) {
	open(IN, "< $message");
	print while (<IN>);
	close(IN);
}

print <<END_HTML;
<p align="center">
<small><a href="http://rock13.com/webhelp/perl/http_errors/">- http_errors.cgi -</a></small></p>
</body>
</html>
END_HTML

exit;



# - - - - - - - - - - - - - - - - - - -
# OTHER FILES
# - - - - - - - - - - - - - - - - - - -
sub other_file {

my $req = shift;

print "<p>Sorry, <code>$req</code> could not be found.</p>\n"
	if $STATUS eq 404;
print "<p>An index is not available for <code>$req</code>.</p>\n"
	if $STATUS eq 403;

print <<HTML;
<h4>Some things that might have went awry, and what you can do.</h4>
<ul>
<li>Check the spelling. Shi- er, typos happen <code>:-)</code></li>
<li>Not all directories have a public index file.</li>
HTML
if($email) {
print qq{<li>If you followed a link from <em>my</em> site email: <a href="mailto:$email">$email</a>.</li>};
}

{
	local $_ = $REFERER;
	if (defined && m#^http://\w+# && ! /$HOST/) {
		print "<li>If you followed a link from someone else's site it ",
		"would be nice to let <a href=\"$REFERER\">them</a> know.</li>\n";
	}
}

print "</ul>\n";

}

# - - - - - - - - - - - - - - - - - - -
# TEMP FILES
# - - - - - - - - - - - - - - - - - - -

sub temp_file {

my $req = shift;

print <<HTML;
<p>Temporary file: <code>$req</code> not found.</p>
<p>Files in the <code>/temp/</code> directory are placed there on a <em>temp</em>orary basis and links to them were never intended to be permanent. You might email me to ask about it: <a href="mailto:$email">$email</a></p>
HTML

}

# - - - - - - - - - - - - - - - - - - -
# IMAGE FILES
# - - - - - - - - - - - - - - - - - - -

sub image_file {
# If the image's parent directory does not exist then this routine never
# gets called.

my $req = shift;
my $bandwidth_url = 'http://google.com/search?q=Bandwidth+Theft';

print <<HTML;
<p>The image (<code>$req</code>) you are trying to access has either moved or been removed. Probably because some twit was hotlinking it -- and <em>that</em> pisses me off. Read up on <a href="$bandwidth_url">Bandwidth Theft</a></p>
HTML

}

# - - - - - - - - - - - - - - - - - - -
# ABOUT ME
# - - - - - - - - - - - - - - - - - - -
sub about_me {
# Print an information page when the script is called directly.

print <<HTML;
<p>Hey there! I'm just a little ol' Perl script that sits here at <a href="http://$HOST/">$HOST</a> and waits for errors to process. Then I do so with all the intelligence and grace I can muster -- isn't that <em>nice</em> of me?</p>
<p>You might read more about me at <a href="http://rock13.com/webhelp/perl/http_errors/">http://rock13.com/webhelp/perl/http_errors/</a>. Of course the owner of $HOST may have modified my considerably.</p>
HTML

}